// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef BATTERYSTATUSWORKER_H
#define BATTERYSTATUSWORKER_H

#include <QMutex>
#include <QObject>
#include <QWaitCondition>

class BatteryStatusWorker : public QObject {
    Q_OBJECT
  public:
    enum Status {
        UNKNOWN,    ///< Cannot determine power status.
        ON_BATTERY, ///< Not plugged in, running on the battery
        NO_BATTERY, ///< Plugged in, no battery available.
        CHARGING,   ///< Plugged in, charging battery
        CHARGED     ///< Plugged in, battery charged
    };
    Q_ENUM(Status)

    BatteryStatusWorker(QObject *parent = nullptr);
    ~BatteryStatusWorker() = default;

    void stop();
    void pause();
    void resume();

    bool isPaused();
    bool isRunning();

    void lastStatus(BatteryStatusWorker::Status *status = nullptr, int *percent = nullptr, int *lifetime = nullptr);
  public slots:
    void process();

  signals:
    void statusUpdated(BatteryStatusWorker::Status status, int percent, int lifetime);
    void finished();

  private:
    mutable QMutex mSyncMutex;
    mutable QMutex mPauseMutex;
    mutable QMutex mDataMutex;
    QWaitCondition mPauseCond;
    volatile bool mPause;
    volatile bool mCancel;
    bool mRunning;

    BatteryStatusWorker::Status mStatus;
    int mPercent;
    int mLifetime;
};

#endif // BATTERYSTATUSWORKER_HPP
