// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef FITCONTROLLER_HPP
#define FITCONTROLLER_HPP

#include "fitworker.hpp"
#include <QObject>
#include <QThread>

class FitController : public QObject {
    Q_OBJECT

  public:
    explicit FitController(QObject *parent = nullptr);
    ~FitController();

    void fitBlock(const ABlock &block, double refTime);
    void setConfiguration(const FitConfig &config,bool fastCalculationMode=false);
    void waitForFinished();
    bool isRunning() const;
public slots:
    void setMaximumInputFrequency(double frequency);
    void setFitProcess(bool enabled);
signals:
    void blockToFit(const ABlock &block, double refTime);
    void fitComplete(const ABlock &block, const QVector<FitReport> &reports, const ExecTimes &times);

  private:
    QThread mWorkerThread;
    FitWorker *mFitWorker;
    bool mEnabled;
    int mMinTimeout;
    QElapsedTimer mFreqTimer;
};

#endif // FITCONTROLLER_HPP
