// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef FITPLOT_HPP
#define FITPLOT_HPP

#include "baseplot.hpp"
#include "lineposition.hpp"
#include "legend.hpp"

class FitPlot : public BasePlot {
    Q_OBJECT
  public:
    FitPlot(QWidget *parent = nullptr);
    virtual ~FitPlot() override;
    void clearData();
    QCPGraph *rawSpectraGraph() { return mRawSpectraGraph; }
    QCPGraph *fitSpectraGraph() { return mFitSpectraGraph; }
    QCPGraph *residualsGraph() { return mResiduGraph; }
    QCPGraph *rampGraph() { return mRampGraph; }

    void setRampIntervalVisible(bool visible);
    void setRampInterval(const QPair<int, int> &range);
    void setSpectraIntervalVisible(bool visible);
    void setSpectraInterval(const QPair<int, int> &range);

  private slots:
    void autoXAxisRange(const QCPRange &newRange);
    void onContextMenuRequested(const QPoint &pos);
    void updateLegends();

  private:
    QCPGraph *mRampGraph;    ///< The initial ramp.
    LinePosition *mBracket1; ///< Lower bound vertical bar for the ramp.
    LinePosition *mBracket2; ///< Upper bound vertical bar for the ramp.

    LinePosition *mBracket3; ///< Lower bound vertical bar for the spectra.
    LinePosition *mBracket4; ///< Upper bound vertical bar for the spectra.

    QCPGraph *mRawSpectraGraph; ///< The raw spectra graph.
    QCPGraph *mFitSpectraGraph; ///< The fit spectra graph
    QCPGraph *mResiduGraph;     ///< The residu graph.

    QVector<Legend *> mLegends;   /// legends.
    QCPMarginGroup *mMarginGroup; ///< Margin group.

  private:
    void addAxisRect();
    void setupGraphs();
    void synchronizeXaxes();
};

#endif // FITPLOT_HPP
