// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "itemtracer.hpp"

ItemTracer::ItemTracer(QCustomPlot *parentPlot) : QCPItemTracer(parentPlot) {}

ItemTracer::~ItemTracer() {}

double ItemTracer::selectTest(const QPointF &pos, bool onlySelectable, QVariant *details) const {
    Q_UNUSED(details)
    if (onlySelectable && !mSelectable)
        return -1;

    QPointF center(position->pixelPosition());
    double w = mSize / 2.0;
    QRect clip = clipRect();

    if (clip.intersects(QRectF(center - QPointF(w, w), center + QPointF(w, w)).toRect())) {
        // distance to border:
        double centerDist = QCPVector2D(center - pos).length();
        double circleLine = w;
        double result = qAbs(centerDist - circleLine);
        // filled ellipse, allow click inside to count as hit:
        if (result > mParentPlot->selectionTolerance() * 0.99 && mBrush.style() != Qt::NoBrush && mBrush.color().alpha() != 0) {
            if (centerDist <= circleLine)
                result = mParentPlot->selectionTolerance() * 0.99;
        }
        return result;
    } else {
        return qSqrt(QCPVector2D(pos)
                         .distanceSquaredToLine(QCPVector2D(center.x(), clip.top()), QCPVector2D(center.x(), clip.bottom())));
    }
}

/* inherits documentation from base class */
void ItemTracer::draw(QCPPainter *painter) {
    updatePosition();
    if (mStyle == tsNone)
        return;

    painter->setPen(mainPen());
    painter->setBrush(mainBrush());
    QPointF center(position->pixelPosition());
    double w = mSize / 2.0;
    QRect clip = clipRect();

    // we draw a circle and two vertical line that span to the top/bottom of the axis rect.

    if (clip.intersects(QRectF(center - QPointF(w, w), center + QPointF(w, w)).toRect()))
        painter->drawEllipse(center, w, w);
    if (center.x() > clip.left() && center.x() < clip.right()) {
        painter->drawLine(QLineF(center.x(), clip.top(), center.x(), center.y() + w));
        painter->drawLine(QLineF(center.x(), center.y() - w, center.x(), clip.bottom()));
    }
}
