// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef LINEPOSITION_HPP
#define LINEPOSITION_HPP

#include "qcustomplot.h"

/**
 * @brief Defines a line (either vertical or horizontal).
 *
 */
class LinePosition : public QCPItemStraightLine {
    Q_OBJECT
  public:
    LinePosition(QCPAxisRect *axisRect, double value = 0.5, Qt::Orientation orientation = Qt::Vertical);
    void setFixed(bool fixed);
    bool isFixed() const;
    void boundToData(bool enable);
    bool boundedToData() const;
    double position() const;
    Qt::CursorShape cursorShape(QMouseEvent *event);

    void setChannelFillLine(LinePosition *targetLine);
    LinePosition *channelFillLine() const;

    QBrush brush() const;
    void setBrush(const QBrush &brush);

    bool antialiasedFill() const;
    void setAntialiasedFill(bool enabled);

  public slots:
    void setPosition(double value);

signals:
    /**
     * @brief Emit the position of the line whenver it has changed (from mouse interactions only).
     * @param value Position value.
     */
    void positionChanged(double value);
    /**
     * @brief Emit the new position of mouse released <b>only</b>.
     * @param value Position value.
     */
    void mouseReleasedPosition(double value);

  protected:
    virtual void mousePressEvent(QMouseEvent *event, const QVariant &details) override;
    virtual void mouseMoveEvent(QMouseEvent *event, const QPointF &startPos) override;
    virtual void mouseReleaseEvent(QMouseEvent *event, const QPointF &startPos) override;
    virtual void move(QPointF pos);

    virtual void draw(QCPPainter *painter) override;
    void drawFill(QCPPainter *painter) const;

    QLineF getLine();
    double axisRectRatio() const;

  private:
    bool mFixed;                             ///< Fixed position control flag.
    bool mBoundToData;                       ///< Line position coordinates system.
    Qt::Orientation mOrientation;            ///< Oritentation of the line.
    QCPAxisRect *mAxisRect;                  ///< Pointer to the Axis Rect.
    bool mIsDraggable;                       ///< Indicate whether or not the plot is (currently) draggable
    bool mDragged;                           ///< Indicate whether or not the line is being dragged.
    QBrush mBrush;                           ///< Brush for channel filling (if any).
    QPointer<LinePosition> mChannelFillLine; ///< Other line position used to draw a channel fill.
    bool mAntialiasedFill;
};

#endif // LINEPOSITION_HPP
