// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef TEXTBOX_HPP
#define TEXTBOX_HPP

#include "qcustomplot.h"

/**
 * @brief The TextBox class
 * @details
 * The subclass stores 6 pre-defined positions (expressed in the AxisRectRatio coordinates) corresponding to the corners the
 * AxisRect and the centers of both top and bottom edges.
 *
 * Upon initialization, the given QCPAxisRect is used to setup the parent plot of the item and the item's position tied
 * AxisRect.
 *
 * Then one can used TextBox::setPositionFlag to change the position of the item. Its alignment will be set accordingly to
 * the new position.
 */
class TextBox : public QCPItemText {
  public:
    enum Position {
        TOP_LEFT = 0,
        TOP_CENTER,
        TOP_RIGHT,
        BOTTOM_LEFT,
        BOTTOM_CENTER,
        BOTTOM_RIGHT,
    };
    explicit TextBox(QCPAxisRect *axisRect);
    TextBox(TextBox::Position initialPosition, QCPAxisRect *axisRect);
    virtual ~TextBox();

    void setPositionFlag(Position position);
    Position positionFlag() const;

  private:
    Position mPosFlag;                                                     ///< Current position flag.
    static const QMap<Position, QPair<QPointF, Qt::Alignment>> gPositions; ///< Map of pre-defined positions.
    void setup(QCPAxisRect *axisRect, TextBox::Position initialPosition = TOP_RIGHT);
};

#endif // TEXTBOX_HPP
