// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef ABSTRACTUDPSOCKET_HPP
#define ABSTRACTUDPSOCKET_HPP

#include <QObject>
#include <QtNetwork>
#include "movingavg.hpp"

class AbstractUDPSocket : public QObject {
    Q_OBJECT
  public:
    explicit AbstractUDPSocket(QObject *parent = nullptr);
    ~AbstractUDPSocket();
    virtual bool connectAndBind(const QString &ip, quint16 port);
    void disconnectSocket(bool log = true);

    QUdpSocket *socket();
    QUdpSocket::SocketState socketState() const;
    bool isBound() const;
    bool isListening() const;
    QString ip() const;
    quint16 port() const;
    const QString &name();
    void setName(const QString &name);


signals:
    void socketStateChanged(QUdpSocket::SocketState state);

    /**
     * @brief Emitted whenever a datagram is processed.
     * @param valid True if the content of the datagram is valid.
     * @param size Size of the current/latest datagram.
     * @details
     * The base class obviously never emit the signal as it should be only performed in
     * AbstractUDPSocket::readPendingDatagrams method.
     */
    void udpStatus(bool valid, int size,qint64 elapsed);

  public slots:
    virtual void setListening(bool enable);
    void setMaximumInputFrequency(double frequency);
  private slots:
    virtual void readPendingDatagrams() = 0;

  protected:
    QUdpSocket *mSocket;
    bool mListen;
    QString mName;
    int mMinTimeout;
    QElapsedTimer mFreqTimer;
    MovingAverage mByteRate;
    MovingAverage mBlockRate;
};

#endif // ABSTRACTUDPSOCKET_HPP
