// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef AUDPSOCKET_HPP
#define AUDPSOCKET_HPP

#include "abstractudpsocket.hpp"
#include "globals.hpp"
#include "movingavg.hpp"

class AUDPSocket : public AbstractUDPSocket {
    Q_OBJECT
  public:
    explicit AUDPSocket(QObject *parent = nullptr);
    virtual ~AUDPSocket() override;
    virtual bool connectAndBind(const QString &ipAddr, quint16 portNum) override;

    bool float32Mode() const;
    void setBlockTemplate(const ABlock &blockTemplate);

    void resetStatistics();
    double blockRate() const { return mBlockRate(); }
    double byteRate() const { return mByteRate(); }
    void getRates(double *block = nullptr, double *bytes = nullptr);
  public slots:

    virtual void setListening(bool enable);
  signals:

    /// @brief Emitted whenever a valid Amulse data block has been received.
    void receiveABlock(const ABlock &block);

    /**
     * @brief Emitted if block's instrument number and/or firmware version do not match the template.
     * @param bInstr Block intrument number.
     * @param bVer Block firmware version.
     * @param tInstr Template intrument number.
     * @param tVer Template firmware version.
     * @note
     * The connected slots should probably stopped any acquistion: block will not be decrypted.
     */
    void invalidTemplate(int bInstr, int bVer, int tInstr, int tVer);

  private slots:
    virtual void readPendingDatagrams() override;

  private:
    ABlock mBlockTemplate;
    bool mNoValid;
};

#endif // AUDPSOCKET_HPP
