// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef BASEMAPSELECTOR_HPP
#define BASEMAPSELECTOR_HPP

#include "globals.hpp"
class ClickableLabel;

struct BasemapInfos {
    BasemapInfos(int id, const QString &path) : id(id), path(path) {}
    int id;       // ID of the basemap: >= 0 for ArcGIS basemaps (enum value), -1 for custom basemap.
    QString path; // Nothing for ArcGIS basemaps, file path for custo/
};

class BasemapSelector : public QScrollArea {
    Q_OBJECT
  public:
    explicit BasemapSelector(QWidget *parent = nullptr);
    explicit BasemapSelector(Qt::Orientation orientation, QWidget *parent = nullptr);

    void selectId(int id, bool signalNeeded = false);
    int selectedId();
    const BasemapInfos &selectedBasemap();
    QString customFilePath() const;
    void translateUI();
    void resetCustomItem();

  signals:
    void selectionChanged(const BasemapInfos &infos);

  private slots:
    bool updateCustomFilePath();
    void onButtonChecked(int buttonId);

  private:
    QBoxLayout *mLayout;
    QWidget *mWidget;
    QButtonGroup *mButtonGroup;
    QVector<QLabel *> mLabels;
    ClickableLabel *mCustomLabel;
    QRadioButton *mCustomButton;
    QVector<QWidget *> mBasemapWidgets;
    QVector<BasemapInfos> mItems;
    int mLastId;
    void setupUI(Qt::Orientation orientation);
    int indexOfId(int id);
    void translateCustom();
};

#endif // BASEMAPSELECTOR_HPP
