// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef CFG_VERSIONS_HPP
#define CFG_VERSIONS_HPP

/**
 * @brief Current ADMin configuration file version.
 * @details
 * The symbol can be used when the configuration file is written to disk.
 * @note
 * Currently it is not used as the version is a member of the relevant class.
 */
#define ADM_FILE_CUR_VER 3

/**
 * @brief Minimum required ADMin  configuration file version.
 * @details
 * The symbol is used when a configuration file is checked for validity.
 * If the version in the configuration is lower than ADM_FILE_MIN_VER, we can expect something
 * wrong will happen.
 */
#define ADM_FILE_MIN_VER 3



/**
 * @brief Current version of \b keymap node.
 * @details
 * The symbol can be used when a \b keymap node is written to disk.
 * @note
 * Currently it is not used as the version is a member of the relevant class.
 */
#define KMP_XML_CUR_VER 1

/**
 * @brief Minimum required version of \b keymap node.
 * @details
 * The symbol is used when a \b keymap node is checked for validity.
 * If the version in the node is lower than KMP_XML_MIN_VER, we can expect something
 * wrong will happen.
 */
#define KMP_XML_MIN_VER 1



/**
 * @brief Current version of \b scenarios node.
 * @details
 * The symbol is used when a scenarios node is written to disk.
 */
#define SCN_XML_CUR_VER 1

/**
 * @brief Minimum required version of \b scenarios nodes.
 * @details
 * The symbol is used when a scenarios node is checked for validity.
 * If the version in the node is lower than SCN_XML_MIN_VER, we can expect something
 * wrong will happen.
 */
#define SCN_XML_MIN_VER 1

/**
 * @brief Current version of \b parameters node.
 * @details
 * The symbol is used when a \b parameters node is written to disk.
 */
#define PRM_XML_CUR_VER 1

/**
 * @brief Minimum required version of \b scenarios node.
 * @details
 * The symbol is used when a \b parameters node is checked for validity.
 * If the version in the node is lower than PRM_XML_MIN_VER, we can expect something
 * wrong will happen.
 */
#define PRM_XML_MIN_VER 1

/**
 * @brief Current version of \b indicators node.
 * @details
 * The symbol is used when a \b indicators node is written to disk.
 */
#define IND_XML_CUR_VER 1

/**
 * @brief Minimum required version of \b indicators node.
 * @details
 * The symbol is used when a \b indicators node is checked for validity.
 * If the version in the node is lower than IND_XML_MIN_VER, we can expect something
 * wrong will happen.
 */
#define IND_XML_MIN_VER 1


#endif // CFG_VERSIONS_HPP
