// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef CLICKABLELABEL_HPP
#define CLICKABLELABEL_HPP

#include <QtWidgets>

/**
 * @brief The ClickableLabel class
 * @details
 * A simple QLabel that is clickable !
 * @note
 * The signal is emitted when the mouse is released inside the bounding box of the label.
 */
class ClickableLabel : public QLabel {
    Q_OBJECT

  public:
    explicit ClickableLabel(QWidget *parent = nullptr);
    virtual ~ClickableLabel() override {}

  public slots:
    void setClickable(bool enable);
    void setHighlight(bool enable);

signals:
    /// @brief Emitted whenever the mouse has been clicked over the label.
    void clicked();
    /// @brief Emitted when the mouse leaves the widget.
    void mouseLeave();
    /// @brief Emitted when the mouse enters the widget.
    void mouseEnter();

  protected:
    virtual void enterEvent(QEvent *event) override;
    virtual void leaveEvent(QEvent *event) override;
    virtual void mouseReleaseEvent(QMouseEvent *event) override;

  private:
    bool mClickable;
    bool mHighlightable;
};

#endif // CLICKABLELABEL_HPP
