// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "colorindicator.hpp"

static const QString greenSS =
    QString("color: white;border-radius: %1;background-color: qlineargradient(spread:pad, x1:0.145, y1:0.16, x2:1, y2:1, "
            "stop:0 rgba(20, 252, 7, 255), stop:1 rgba(25, 134, 5, 255));");
static const QString redSS =
    QString("color: white;border-radius: %1;background-color: qlineargradient(spread:pad, x1:0.145, y1:0.16, x2:0.92, "
            "y2:0.988636, stop:0 rgba(255, 12, 12, 255), stop:0.869347 rgba(103, 0, 0, 255));");
static const QString orangeSS = QString("color: white;border-radius: %1;border-style: outset;background-color: "
                                        "qlineargradient(spread:pad, x1:0.232, y1:0.272, x2:0.98, "
                                        "y2:0.959773, stop:0 rgba(255, 113, 4, 255), stop:1 rgba(91, 41, 7, 255))");
static const QString blueSS = QString("color: white;border-radius: %1;background-color: qlineargradient(spread:pad, x1:0.04, "
                                      "y1:0.0565909, x2:0.799, y2:0.795, stop:0 rgba(203, 220, 255, 255), stop:0.41206 rgba(0, "
                                      "115, 255, 255), stop:1 rgba(0, 49, 109, 255));");
static const QString greySS =
    QString("color: white;border-radius: %1;background-color: qlineargradient(spread:pad, x1:0.145, y1:0.16, x2:1, y2:1, "
            "stop:0 rgba(173, 171, 171, 255), stop:0.6 rgba(138, 138, 138, 255), stop:1 rgba(56, 53, 53, 255));");

/// @brief Base constructor
/// @details Initialize a ColorIndicator with a fixed size of 24 pixels and green color.
ColorIndicatorSS::ColorIndicatorSS(QWidget *parent) : QLabel(parent) {
    setFixedSize(24, 24);
    mGreenSS = greenSS.arg(12);
    mOrangeSS = orangeSS.arg(12);
    mRedSS = redSS.arg(12);
    mBlueSS = blueSS.arg(12);
    mGreySS = greySS.arg(12);
    mColor = StatusColor::Green;
    setStyleSheet(mGreenSS);
}

/**
 * @brief Constructor;
 * @param size Size in pixels of the widget.
 * @param parent Parent widget.
 */
ColorIndicatorSS::ColorIndicatorSS(int size, QWidget *parent) : QLabel(parent) {
    size = qMax(size, 2);
    setFixedSize(size, size);
    mGreenSS = greenSS.arg(size / 2);
    mOrangeSS = orangeSS.arg(size / 2);
    mRedSS = redSS.arg(size / 2);
    mBlueSS = blueSS.arg(size / 2);
    mGreySS = greySS.arg(size / 2);
    mColor = StatusColor::Green;
    setStyleSheet(mGreenSS);
}

/**
 * @brief Set a new color for the indicator.
 * @param color Color to set.
 */
void ColorIndicatorSS::setColor(StatusColor color) {
    if (mColor != color) {
        mColor = color;
        switch (mColor) {
        case StatusColor::Green:
            setStyleSheet(mGreenSS);
            break;
        case StatusColor::Orange:
            setStyleSheet(mOrangeSS);
            break;
        case StatusColor::Red:
            setStyleSheet(mRedSS);
            break;
        case StatusColor::Blue:
            setStyleSheet(mBlueSS);
            break;
        case StatusColor::Grey:
        default:
            setStyleSheet(mGreySS);
        }
    }
    return;
}

/// @brief Get the current color.
StatusColor ColorIndicatorSS::color() const { return mColor; }

/// @brief Base constructor
/// @details Initialize a ColorIndicator with a fixed size of 24 pixels and green color.
ColorIndicator::ColorIndicator(QWidget *parent) : QLabel(parent) {
    setFixedSize(24, 24);

    mGreenPix = QPixmap(":/adm2/images/green-button.png");
    mRedPix = QPixmap(":/adm2/images/red-button.png");
    mYellowPix = QPixmap(":/adm2/images/yellow-button.png");
    mGreyPix = QPixmap(":/adm2/images/gray-button.png");
    mBluePix = QPixmap(":/adm2/images/blue-button.png");

    mColor = StatusColor::Green;
    setPixmap(mGreenPix);
}

/**
 * @brief Constructor;
 * @param size Size in pixels of the widget.
 * @param parent Parent widget.
 */
ColorIndicator::ColorIndicator(int size, QWidget *parent) : QLabel(parent) {
    size = qMax(size, 2);
    setFixedSize(size, size);

    QSize qsize(size, size);

    mGreenPix = QPixmap(":/adm2/images/green-button.png").scaled(qsize);
    mRedPix = QPixmap(":/adm2/images/red-button.png").scaled(qsize);
    mYellowPix = QPixmap(":/adm2/images/yellow-button.png").scaled(qsize);
    mGreyPix = QPixmap(":/adm2/images/gray-button.png").scaled(qsize);
    mBluePix = QPixmap(":/adm2/images/blue-button.png").scaled(qsize);

    mColor = StatusColor::Green;
    setPixmap(mGreenPix);
}

/**
 * @brief Set a new color for the indicator.
 * @param color Color to set.
 */
void ColorIndicator::setColor(StatusColor color) {
    if (mColor != color) {
        mColor = color;
        switch (mColor) {
        case StatusColor::Green:
            setPixmap(mGreenPix);
            break;
        case StatusColor::Orange:
            setPixmap(mYellowPix);
            break;
        case StatusColor::Red:
            setPixmap(mRedPix);
            break;
        case StatusColor::Blue:
            setPixmap(mBluePix);
            break;
        case StatusColor::Grey:
        default:
            setPixmap(mGreyPix);
        }
    }
    return;
}

/// @brief Get the current color.
StatusColor ColorIndicator::color() const { return mColor; }
