// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "amdataset.hpp"
#include "fileloader.hpp"
#include "dataloaderdialog.hpp"
#include "dataloader.hpp"
#include "itemmapper.hpp"
#include "prm_utils.hpp"

DataLoaderDialog::DataLoaderDialog(const QStringList &files, const ItemMapper *mapper, QWidget *parent,
                                   Qt::WindowFlags flags)
    : QDialog(parent, flags) {

    QSettings settings;
    mMaxPoints = settings.value("replay/max_points", 100000).toInt();

    mItemMapper = mapper;
    mReplayData = nullptr;
    mDataset = nullptr;
    mFiles = files;

    QVBoxLayout *layout;

    mGbxProgress = new QGroupBox(tr("Progression"), this);
    mLblProgressStep = new QLabel(mGbxProgress);
    mLblProgressInfo = new QLabel(mGbxProgress);
    layout = new QVBoxLayout(mGbxProgress);
    layout->addWidget(mLblProgressStep);
    layout->addWidget(mLblProgressInfo);
    layout->addStretch(0);

    mLblProgressStep->setText(tr("Step: No operations in progress"));
    mLblProgressInfo->setText(tr("Info: No messages."));

    mGbxDatasetInfos = new QGroupBox(tr("Dataset informations"), this);
    mLblDINb = new QLabel("N/A", mGbxDatasetInfos);
    mLblStartDate = new QLabel("N/A", mGbxDatasetInfos);
    mLblEndDate = new QLabel("????-??-??, ??:??:??", mGbxDatasetInfos);
    mLblTimestep = new QLabel("????-??-??, ??:??:??", mGbxDatasetInfos);
    QString dt = QString("%1t").arg(QChar(0x0394));
    QGridLayout *glayout = new QGridLayout(mGbxDatasetInfos);
    glayout->addWidget(new QLabel(tr("Blocks"), mGbxDatasetInfos), 0, 0);
    glayout->addWidget(mLblDINb, 0, 1);
    glayout->addWidget(new QLabel(QString("<%1>").arg(dt), mGbxDatasetInfos), 0, 2);
    glayout->addWidget(mLblTimestep, 0, 3);
    glayout->addWidget(new QLabel(tr("Start"), mGbxDatasetInfos), 1, 0);
    glayout->addWidget(mLblStartDate, 1, 1);
    glayout->addWidget(new QLabel(tr("End"), mGbxDatasetInfos), 1, 2);
    glayout->addWidget(mLblEndDate, 1, 3);
    glayout->setColumnStretch(0, 0);
    glayout->setColumnStretch(1, 1);
    glayout->setColumnStretch(2, 0);
    glayout->setColumnStretch(3, 1);
    glayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding), 3, 0, 1, -1);

    //    QFormLayout *flayout = new QFormLayout(mGbxDatasetInfos);
    //    flayout->addRow(QString("<%1>").arg(dt), mLblTimestep);
    //    flayout->addRow(tr("Blocks"), mLblDINb);
    //    flayout->addRow(tr("Start"), mLblStartDate);
    //    flayout->addRow(tr("End"), mLblEndDate);

    mGbxOperations = new QGroupBox(tr("Operation"), this);
    mRdbNothing = new QRadioButton(tr("Do not modify."), mGbxOperations);
    mRdbDecimate = new QRadioButton(tr("Decimate."), mGbxOperations);
    mSpbDecimate = new QSpinBox(mGbxOperations);
    mSpbDecimate->setPrefix(tr("Factor: "));
    mSpbDecimate->setEnabled(false);
    mSpbDecimate->setAccelerated(true);
    mSpbDecimate->setStepType(QSpinBox::AdaptiveDecimalStepType);
    QObject::connect(mRdbDecimate, &QRadioButton::toggled, mSpbDecimate, &QWidget::setEnabled);
    mRdbDownSample = new QRadioButton(tr("Resample."), mGbxOperations);
    mSpbDownSample = new QDoubleSpinBox(mGbxOperations);
    mSpbDownSample->setPrefix(QString("%1t: ").arg(QChar(0x0394)));
    mSpbDownSample->setDecimals(3);
    mSpbDownSample->setSuffix(" sec");
    mSpbDownSample->setEnabled(false);
    mSpbDownSample->setAccelerated(true);
    mSpbDownSample->setStepType(QSpinBox::AdaptiveDecimalStepType);
    QObject::connect(mRdbDownSample, &QRadioButton::toggled, mSpbDownSample, &QWidget::setEnabled);
    mRdbTruncate = new QRadioButton(tr("Truncate."));
    mSpbTruncate = new QSpinBox(mGbxOperations);
    mSpbTruncate->setPrefix(tr("keep: "));
    mSpbTruncate->setSuffix(tr(" points"));
    mSpbTruncate->setEnabled(false);
    mSpbTruncate->setAccelerated(true);
    mSpbTruncate->setStepType(QSpinBox::AdaptiveDecimalStepType);
    QObject::connect(mRdbTruncate, &QRadioButton::toggled, mSpbTruncate, &QWidget::setEnabled);

    mRdbNothing->setChecked(true);

    // QGridLayout *glayout;
    glayout = new QGridLayout(mGbxOperations);
    glayout->addWidget(mRdbNothing, 0, 0, 1, 2);
    glayout->addWidget(getSeparator(mGbxOperations), 1, 0, 1, 2);
    glayout->addWidget(mRdbDecimate, 2, 0);
    glayout->addWidget(mSpbDecimate, 2, 1);
    glayout->addWidget(mRdbDownSample, 3, 0);
    glayout->addWidget(mSpbDownSample, 3, 1);
    glayout->addWidget(mRdbTruncate, 4, 0);
    glayout->addWidget(mSpbTruncate, 4, 1);
    glayout->setColumnStretch(0, 0);
    glayout->setColumnStretch(1, 1);
    glayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding), 5, 0, 1, -1);

    //    layout = new QVBoxLayout(mGbxOperations);
    //    layout->addWidget(mRdbNothing);
    //    layout->addWidget(getSeparator(mGbxOperations));
    //    QHBoxLayout *hlayout;
    //    hlayout = new QHBoxLayout;
    //    hlayout->setContentsMargins(0, 0, 0, 0);
    //    hlayout->addWidget(mRdbDecimate, 1);
    //    hlayout->addWidget(mSpbDecimate, 0, Qt::AlignRight);
    //    layout->addLayout(hlayout);
    //    hlayout = new QHBoxLayout;
    //    hlayout->setContentsMargins(0, 0, 0, 0);
    //    hlayout->addWidget(mRdbDownSample, 1);
    //    hlayout->addWidget(mSpbDownSample, 0, Qt::AlignRight);
    //    layout->addLayout(hlayout);
    //    hlayout = new QHBoxLayout;
    //    hlayout->setContentsMargins(0, 0, 0, 0);
    //    hlayout->addWidget(mRdbTruncate, 1);
    //    hlayout->addWidget(mSpbTruncate, 0, Qt::AlignRight);
    //    layout->addLayout(hlayout);
    //    layout->addStretch(0);

    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel,this);

    mOkButton = buttonBox->button(QDialogButtonBox::Ok);
    mCancelButton = buttonBox->button(QDialogButtonBox::Cancel);

    // dialog layout

    layout = new QVBoxLayout(this);
    layout->addWidget(mGbxProgress, 0, Qt::AlignTop);
    layout->addWidget(mGbxDatasetInfos, 0, Qt::AlignTop);
    layout->addWidget(mGbxOperations, 0, Qt::AlignTop);
    layout->addStretch(1);
    layout->addWidget(buttonBox);

    // disable some widgets
    mGbxOperations->setEnabled(false);
    mRdbNothing->setEnabled(false);
    mOkButton->setEnabled(false);

    QObject::connect(mCancelButton, &QPushButton::clicked, this, &DataLoaderDialog::cancelAndQuit);

    setMaximumWidth(800);
}

/// @brief Destructor.
DataLoaderDialog::~DataLoaderDialog() {
    if (mDataset)
        delete mDataset;
}

/**
 * @brief Get the map of data.
 * @return Pointer to the generated replay data.
 * @details
 * The method returns the replay data structure generated by the internal data loader.
 * The pointer may be null if the loader did not succeed. Otherwise, it is not owned by the dialog and must be
 * deleted by user when no more used !
 */
ReplayData *DataLoaderDialog::data() const { return mReplayData; }

/// @brief Cancel job and quit.
void DataLoaderDialog::cancelAndQuit() {
    if (mFileLoader)
        mFileLoader->cancel();
    else if (mDataLoader)
        mDataLoader->cancel();
    else
        reject();
}

/**
 * @brief Setup the file loader on show event.
 * @param event Associated show event.
 */
void DataLoaderDialog::showEvent(QShowEvent *event) {
    QTimer::singleShot(50, this, &DataLoaderDialog::loadFiles);
    QDialog::showEvent(event);
}

/// @brief Get the size hint for this dialog.
QSize DataLoaderDialog::sizeHint() const {
    QSize sz(minimumSize());
    sz.rwidth() = qBound(250, sz.rwidth(), 800);
    return sz;
}

/// @brief Update labels on progress changed.
void DataLoaderDialog::onFileProgressChanged(int index) {
    int nf = mFiles.size();
    if (index >= nf) {
        mLblProgressStep->setText(tr("Step: Loading files (%1/%1).").arg(nf));
        mLblProgressInfo->setText(tr("Info: All files parsed."));
    } else {
        mLblProgressStep->setText(tr("Step: Loading files (%1/%2).").arg(index + 1).arg(nf));
        mLblProgressInfo->setText(tr("Info: Reading %1.").arg(QFileInfo(mFiles.at(index)).fileName()));
    }
}

/**
 * @brief Manage dialog interactions on file loading complete.
 */
void DataLoaderDialog::onFileLoadingFinished() {
    bool canceled = mFileLoader->canceled();
    mLoaderThread->deleteLater();
    mLoaderThread = nullptr;
    QObject::disconnect(mFileLoader, &FileLoader::message, mLblProgressInfo, &QLabel::setText);
    QObject::disconnect(mFileLoader, &FileLoader::progressChanged, this, &DataLoaderDialog::onFileProgressChanged);
    mLblProgressInfo->setText(tr("Info: loading finished."));
    mFileLoader->deleteLater();
    mFileLoader = nullptr;
    if (canceled) {
        QDialog::reject();
        return;
    }
    checkDataset();
}

/// @brief Load files into a new dataset.
void DataLoaderDialog::loadFiles() {
    Q_ASSERT_X(!mFileLoader, "DataLoaderDialog::loadFiles", "Internal file loader should not be initialized yet !!");
    if (mFiles.isEmpty()) {
        reject();
        return;
    }
    mFileLoader = new FileLoader(mFiles);
    mLoaderThread = new QThread;
    mFileLoader->moveToThread(mLoaderThread);
    QObject::connect(mLoaderThread, &QThread::started, mFileLoader, &FileLoader::process);
    QObject::connect(mFileLoader, &FileLoader::message, mLblProgressInfo, &QLabel::setText);
    QObject::connect(mFileLoader, &FileLoader::progressChanged, this, &DataLoaderDialog::onFileProgressChanged);
    QObject::connect(mFileLoader, &FileLoader::datasetLoaded, this, &DataLoaderDialog::onDatasetLoaded);
    QObject::connect(mFileLoader, &FileLoader::finished, mLoaderThread, &QThread::quit);
    QObject::connect(mLoaderThread, &QThread::finished, this, &DataLoaderDialog::onFileLoadingFinished);
    mLoaderThread->start();
}

/// @brief get the pointer of the dataset generated by the file loader.
void DataLoaderDialog::onDatasetLoaded(AMDataset *dataset) { mDataset = dataset; }

/// @brief Cleanup loader and close dialog on data processing finished.
void DataLoaderDialog::onDataProcessingFinished() {
    bool canceled = mDataLoader->canceled();
    mLoaderThread->deleteLater();
    mLoaderThread = nullptr;
    mDataLoader->disconnect(mLblProgressInfo);
    mDataLoader->disconnect(this);
    mDataLoader->deleteLater();
    mDataLoader = nullptr;
    if (canceled) {
        QDialog::reject();
        return;
    } else {
        mLblProgressInfo->setText(tr("Info: data extraction finished."));
        QTimer::singleShot(200, this, &QDialog::accept);
    }
}

/// @brief Check the dataset and setup the processing groupbox.
void DataLoaderDialog::checkDataset() {
    if (!mDataset)
        return;
    int nbr = mDataset->blockCount();
    double avgTimeStep = 0.0;
    mLblProgressStep->setText(tr("Step: Checking input dataset."));
    // Setup dataset informations:
    mLblDINb->setText(QString::number(nbr));
    if (nbr) {
        mLblStartDate->setText(
            QDateTime::fromSecsSinceEpoch(mDataset->startTime(), Qt::UTC).toString("yyyy-MM-dd, HH:mm:ss"));
        mLblEndDate->setText(
            QDateTime::fromSecsSinceEpoch(mDataset->endTime(), Qt::UTC).toString("yyyy-MM-dd, HH:mm:ss"));
        for (ADataset *d : mDataset->datasets())
            avgTimeStep += d->refStep() * d->blockCount(); // yeah again assume dataset is ref prm is time.
        avgTimeStep /= nbr;
        mLblTimestep->setText(QString("%1 seconds").arg(avgTimeStep, 0, 'f', 3));
    } else {
        mLblStartDate->setText("N/A");
        mLblEndDate->setText("N/A");
        mLblTimestep->setText("N/A");
        mLblProgressInfo->setText(tr("Error: No blocks in dataset."));
        return;
    }

    // check dataset parameters against the item mapper.
    AParam nsaPrm = mItemMapper->parameter(ItemMapper::NB_SATS);
    AParam lonPrm = mItemMapper->parameter(ItemMapper::LONGITUDE);
    AParam latPrm = mItemMapper->parameter(ItemMapper::LATITUDE);
    AParam altPrm = mItemMapper->parameter(ItemMapper::ALTITUDE);
    bool hasNsa = mDataset->hasParameter(nsaPrm);
    bool hasLon = mDataset->hasParameter(lonPrm);
    bool hasLat = mDataset->hasParameter(latPrm);
    bool hasAlt = mDataset->hasParameter(altPrm);
    bool hasPcT = mDataset->hasParameter(mItemMapper->parameter(ItemMapper::PCMON_TIME));
    bool hasSeT = mDataset->hasParameter(mItemMapper->parameter(ItemMapper::SENSOR_TIME));

    if (!(hasPcT || hasSeT)) {
        mLblProgressInfo->setText("Error: Cannot extract timeline from dataset.");
        return;
    } else if (!(hasLon && hasLat && hasAlt && hasNsa)) {
        mLblProgressInfo->setText("Error: Cannot extract geographic positions from dataset.");
        return;
    }

    // preprocess: compute a list of valid indexes in the dataset where:
    // nbsats > 3 AND time values not nan.
    AParam timePrm = mItemMapper->parameter(hasPcT ? ItemMapper::PCMON_TIME : ItemMapper::SENSOR_TIME);
    DoubleVector times = mDataset->parameter(timePrm);
    DoubleVector nsats = mDataset->parameter(nsaPrm);

    int ni = nsats.size();
    mBaseIndexes.reserve(ni);
    for (long i = 0; i < ni; i++) {
        double vs = nsats.at(i), vt = times.at(i);
        if (qIsNaN(vs) || vs < 4 || qIsNaN(vt))
            continue;
        mBaseIndexes.push_back(i);
    }
    mBaseIndexes.squeeze();
    int nb = mBaseIndexes.size();
    mLblDINb->setText(QString("%1, valid: %2").arg(nbr).arg(nb));
    if (!nb) {
        mLblProgressInfo->setText("Error: No valid data points in dataset.");
        return;
    }

    DoubleVector tmp;
    tmp.resize(nb);
    for (int i = 0; i < nb; i++)
        tmp[i] = times.at(mBaseIndexes.at(i));

    bool skipEnable = nb <= mMaxPoints;
    if (skipEnable) {
        mRdbNothing->setEnabled(true);
        mRdbNothing->setChecked(true);
        mSpbDecimate->setRange(1, nb - 1);
        mSpbDecimate->setValue(1);

        mSpbDownSample->setRange(avgTimeStep * 1.0001, tmp.back() - tmp.front());
        mSpbDownSample->setValue(mSpbDownSample->minimum());

        mSpbTruncate->setRange(0, nb - 1);
        mSpbTruncate->setValue(0);
    } else {
        mRdbNothing->setEnabled(false);
        mRdbDecimate->setChecked(true);
        int minDS = qMax(1, qCeil(nb * 1.0 / mMaxPoints));
        mSpbDecimate->setRange(minDS, nb - 1);
        mSpbDecimate->setValue(minDS);

        double minRsp = getMinRspStep(mMaxPoints, tmp, &avgTimeStep);
        mSpbDownSample->setRange(minRsp, tmp.back() - tmp.front());
        mSpbDownSample->setValue(mSpbDownSample->minimum());
        mSpbTruncate->setRange(0, mMaxPoints);
        mSpbTruncate->setValue(mMaxPoints);
    }

    //    for (QLabel *l : findChildren<QLabel *>())
    //        l->setText("");

    // updateGeometry();
    mGbxOperations->setEnabled(true);
    mOkButton->setEnabled(true);
    mLblProgressInfo->setText(tr("Info: Dataset validated."));
    QObject::connect(mOkButton, &QPushButton::clicked, this, &DataLoaderDialog::processData);
}

void DataLoaderDialog::processData() {
    Q_ASSERT_X(!mDataLoader, "DataLoaderDialog::processData", "Internal file loader should not be initialized yet !!");
    Q_ASSERT_X(!mLoaderThread, "DataLoaderDialog::processData", "Internal loader thread should be null !!");
    DPP procPrms;
    if (mRdbNothing->isChecked()) {
        procPrms.process = DPP::NOTHING;
    } else if (mRdbDecimate->isChecked()) {
        procPrms.process = DPP::DECIMATE;
    } else if (mRdbDownSample->isChecked()) {
        procPrms.process = DPP::RESAMPLE;
    } else if (mRdbTruncate->isChecked()) {
        procPrms.process = DPP::TRUNCATE;
    } else
        return;
    procPrms.decimationStep = mSpbDecimate->value();
    procPrms.resampleStep = mSpbDownSample->value();
    procPrms.keepPoints = mSpbTruncate->value();

    mDataLoader = new DataLoader(procPrms, mBaseIndexes, mItemMapper, mDataset);
    mLoaderThread = new QThread;
    mDataLoader->moveToThread(mLoaderThread);
    QObject::connect(mLoaderThread, &QThread::started, mDataLoader, &DataLoader::process);
    QObject::connect(mDataLoader, &DataLoader::stepMessage, mLblProgressStep, &QLabel::setText);
    QObject::connect(mDataLoader, &DataLoader::infoMessage, mLblProgressInfo, &QLabel::setText);
    QObject::connect(mDataLoader, &DataLoader::dataReady, this, &DataLoaderDialog::onDataReady);
    QObject::connect(mDataLoader, &DataLoader::finished, mLoaderThread, &QThread::quit);
    QObject::connect(mLoaderThread, &QThread::finished, this, &DataLoaderDialog::onDataProcessingFinished);
    mLoaderThread->start();
    mOkButton->setEnabled(false);
}

/// Store replay data on process finished.
void DataLoaderDialog::onDataReady(ReplayData *replayData) { mReplayData = replayData; }

/// @brief Manage (enter) key press.
void DataLoaderDialog::keyPressEvent(QKeyEvent *e) {
    auto k = e->key();
    // fuckin hack so the dialog won't be accepted !!!
    if (k == Qt::Key_Return || k == Qt::Key_Enter)
        return;
    QDialog::keyPressEvent(e);
}
