// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef DATARANGE_HPP
#define DATARANGE_HPP

#include "globals.hpp"

class DataRange {
  public:
    DataRange();
    DataRange(double lower, double upper);

    static DataRange fromQCPRange(const QCPRange &range);

    bool operator==(const DataRange &other) const;
    bool operator!=(const DataRange &other) const;

    bool contains(double value) const;
    double lower() const;
    double upper() const;

    void setLower(double lower);
    void setUpper(double upper);
    void setRange(double lower, double upper);
    void setRange(const QCPRange &range);

    void updateUnits(Units::Unit unit);
    bool isFinite() const;
    QCPRange toQCPRange() const;
    void normalize();

  private:
    double mLower;
    double mUpper;
};

Q_DECLARE_TYPEINFO(DataRange, Q_MOVABLE_TYPE);

#endif // DATARANGE_HPP
