// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "lineeditdelegate.hpp"
#include <QLineEdit>
#include "validators.hpp"

LineEditDelegate::LineEditDelegate(bool allowEmpty, QObject *parent) : QStyledItemDelegate(parent) { mAllowEmpty = allowEmpty; }

/// @brief Create the widget editor for the given index.
QWidget *LineEditDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    Q_UNUSED(option)
    if (!index.isValid())
        return nullptr;
    QString value = index.model()->data(index, Qt::EditRole).toString();
    QLineEdit *editor = new QLineEdit(parent);
    editor->setText(value);
    return editor;
}

/// @brief Set data in the editor from the given index.
void LineEditDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const {
    if (!index.isValid())
        return;
    QString value = index.model()->data(index, Qt::EditRole).toString();
    if (value.isEmpty() && !mAllowEmpty)
        value = "???";
    QLineEdit *myEditor = qobject_cast<QLineEdit *>(editor);
    if (myEditor) {
        myEditor->setText(value);
        myEditor->selectAll();
    }
}

/// @brief Set model data at index related to the given editor.
void LineEditDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const {
    if (!index.isValid())
        return;
    QLineEdit *myEditor = qobject_cast<QLineEdit *>(editor);
    if (myEditor)
        model->setData(index, myEditor->text());
}

/// @brief Update the geometry of the editor.
void LineEditDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option,
                                            const QModelIndex &index) const {
    QStyledItemDelegate::updateEditorGeometry(editor, option, index);
}
