// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "valuelineeditdelegate.hpp"
#include "validators.hpp"
#include "valuelineedit.hpp"
#include <QLineEdit>

ValueLineEditDelegate::ValueLineEditDelegate(bool allowEmpty, bool allowNaN, QObject *parent)
    : QStyledItemDelegate(parent)
    , fmt('g')
    , prec(6) {
    mAllowEmpty = allowEmpty;
    mAllowNaN = allowNaN;
}

/// @brief Create the widget editor for the given index.
QWidget *ValueLineEditDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    Q_UNUSED(option)
    if (!index.isValid())
        return nullptr;
    double value = index.model()->data(index, Qt::EditRole).toDouble();

    QLineEdit *editor = new ValueLineEdit(mAllowEmpty, mAllowNaN, parent);

    //    QLineEdit *editor = new QLineEdit(parent);
    // editor->setValidator(new DoubleValidator(editor, mAllowEmpty, mAllowNaN));

    editor->setText(QString::number(value, fmt.toLatin1(), prec));
    return editor;
}

/// @brief Set data in the editor from the given index.
void ValueLineEditDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const {
    if (!index.isValid())
        return;
    bool ok;
    double value = index.model()->data(index, Qt::EditRole).toDouble(&ok);
    if (!ok)
        value = qQNaN();
    QLineEdit *myEditor = qobject_cast<QLineEdit *>(editor);
    if (myEditor) {
        QString text;
        if (qIsNaN(value)) {
            if (mAllowEmpty) {
                text = "";
            } else if (mAllowNaN) {
                text = "nan";
            } else {
                text = "0";
            }
        } else {
            text = QString::number(value, fmt.toLatin1(), prec);
        }
        myEditor->setText(text);
        myEditor->selectAll();
    }
}

/// @brief Set model data at index related to the given editor.
void ValueLineEditDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const {
    if (!index.isValid())
        return;
    QLineEdit *myEditor = qobject_cast<QLineEdit *>(editor);
    if (myEditor) {
        bool ok;
        QString s = myEditor->text();
        double v = s.toDouble(&ok);
        if (!ok)
            v = qQNaN();
        model->setData(index, v);
    }
}

/// @brief Update the geometry of the editor.
void ValueLineEditDelegate::updateEditorGeometry(QWidget *editor,
                                                 const QStyleOptionViewItem &option,
                                                 const QModelIndex &index) const {
    QStyledItemDelegate::updateEditorGeometry(editor, option, index);
}
