// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef DNDVIEWSTYLE_HPP
#define DNDVIEWSTYLE_HPP

#include <QProxyStyle>
#include <QStyleOption>

/**
 * @brief The DnDViewStyle class
 * @details
 * The class reimplements the drawPrimitive method in order to force the drop indicator (on item view) to span all columns of
 * the view and not only the current cell.
 */
class DnDViewStyle : public QProxyStyle {
  public:
    virtual void drawPrimitive(QStyle::PrimitiveElement element, const QStyleOption *option, QPainter *painter,
                               const QWidget *widget = nullptr) const override {

        if (element == QProxyStyle::PE_IndicatorItemViewItemDrop && option && !option->rect.isNull()) {
            QStyleOption newOption(*option);
            newOption.rect.setLeft(0);
            if (widget)
                newOption.rect.setRight(widget->width());
            QProxyStyle::drawPrimitive(element, &newOption, painter, widget);
        } else {
            QProxyStyle::drawPrimitive(element, option, painter, widget);
        }
   }
};

#endif // DNDVIEWSTYLE_HPP
