// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "filenamewidget.hpp"

/**
 * @brief Constructor.
 * @param parent Optional parent widget pointer.
 */
FilenameWidget::FilenameWidget(QWidget *parent) : QWidget(parent) {
    mNoSD = false;
    mLblSensorDescr = new QLabel(this);
    mLblSensorValue = new QLabel(this);
    mLblAdminDescr = new QLabel(this);
    mLblAdminValue = new QLabel(this);

    mLblSensorValue->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
    mLblAdminValue->setAlignment(Qt::AlignVCenter | Qt::AlignRight);

    QFormLayout *layout = new QFormLayout(this);
    layout->addRow(mLblSensorDescr, mLblSensorValue);
    layout->addRow(mLblAdminDescr, mLblAdminValue);

    mLblSensorValue->setText("N/A");
    mLblAdminValue->setText("N/A");
    translateUI();
}

/// @brief Set new filename for sensor.
void FilenameWidget::setSensorFileName(const QString &name, bool nosd) {
    mLblSensorValue->setText(name);
    if (nosd != mNoSD) {
        mNoSD = nosd;
        setTextHighlight(nosd);
    }
}

void FilenameWidget::setTextHighlight(bool warn) {
    QColor color(warn ? Qt::red : Qt::black);
    for (QLabel *lbl : {mLblSensorDescr, mLblSensorValue}) {
        QFont font = lbl->font();
        font.setBold(warn);
        lbl->setFont(font);
        QPalette pal = lbl->palette();
        pal.setColor(QPalette::WindowText, color);
        lbl->setPalette(pal);
    }
}

/// @brief Set new filename for Admin.
void FilenameWidget::setADMinFileName(const QString &name) { mLblAdminValue->setText(name); }

/// @brief Ge the size hint for this widget.
QSize FilenameWidget::sizeHint() const { return QSize(150, 60); }

/// @brief Translate widgets texts.
void FilenameWidget::translateUI() {
    setWindowTitle(tr("Filenames"));
    mLblSensorDescr->setText(tr("Sensor"));
    mLblAdminDescr->setText(tr("ADMin"));
}

/// @brief Reset the widget to its initial state.
void FilenameWidget::reset() {
    setADMinFileName("N/A");
    setSensorFileName("N/A");
    setTextHighlight(false);
    mNoSD = false;
}
