// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "gpsstatuswidget.hpp"

#include "colorindicator.hpp"

GPSStatusWidget::GPSStatusWidget(QWidget *parent) : QWidget(parent) {

    // udp part
    mIndicator = new ColorIndicator(this);
    mIndicator->setColor(StatusColor::Grey);

    mLblDescr = new QLabel(this);
    mLblValue = new QLabel(this);
    QHBoxLayout *layout = new QHBoxLayout(this);

    layout->addWidget(mIndicator, 0, Qt::AlignLeft);
    layout->addWidget(mLblDescr, 2, Qt::AlignLeft);
    layout->addWidget(mLblValue, 0, Qt::AlignRight);
    translateUI();

    reset();
}

/// @brief Destructor.
GPSStatusWidget::~GPSStatusWidget() {}

/// @brief Translate widgets texts.
void GPSStatusWidget::translateUI() { mLblDescr->setText(tr("GPS status")); }
/**
 * @brief Update (udp) widget from given arguments values.
 * @param valid Control flag indicating whether or not the incoming datagram have been correctly parsed by the application.
 * @param size Size of the (last) incoming datagram.
 */
void GPSStatusWidget::updateStatus(int nsats) {
    mIndicator->setColor(nsats < 4 ? StatusColor::Red : StatusColor::Green);
    mLblValue->setText(QString("%1 satellite(s)").arg(nsats));
}

/// @brief Reset widget to display defaults.
void GPSStatusWidget::reset() {
    mIndicator->setColor(StatusColor::Grey);
    mLblValue->setText("0 satellite(s)");
}
