// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef GPSWIDGET_HPP
#define GPSWIDGET_HPP

#include "globals.hpp"

class ItemMapper;

/**
 * @brief The GPSWidget class
 * @details
 * A simple widget that display GPS status values: longitude, latitude, altitude and number of satellites.
 *
 * Values are updated from the associated item mapper by calling GPSWidget::updateFromMapper.
 */
class GPSWidget : public QWidget {
    Q_OBJECT
  public:
    explicit GPSWidget(QWidget *parent = nullptr);
    void setMapper(ItemMapper *mapper);
    void translateUI();
    void reset();
    void setMinSat(int nsat);
    virtual QSize sizeHint() const override;

  public slots:
    void updateFromMapper();

  private:
    QLabel *mLblNbSatDescr;
    QLabel *mLblLatDescr;
    QLabel *mLblLonDescr;
    QLabel *mLblAltDescr;

    QLabel *mLblNbSatValue;
    QLabel *mLblLatValue;
    QLabel *mLblLonValue;
    QLabel *mLblAltValue;

    QPointer<ItemMapper> mMapper;
    QColor mLastColor;
    int mMinSat;

  private:
    void updateTextColor();
};

#endif // GPSWIDGET_HPP
