// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef INDICATORMODEL_HPP
#define INDICATORMODEL_HPP

#include "globals.hpp"
#include "indicators.hpp"

class ItemMapper;

class IndicatorModel : public QAbstractTableModel {
    Q_OBJECT

  public:
    explicit IndicatorModel(QObject *parent = nullptr);
    IndicatorModel(const QVector<BaseIndicator *> &items, QObject *parent = nullptr);
    ~IndicatorModel();
    void translateHeaders();

    virtual Qt::ItemFlags flags(const QModelIndex &index) const override;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    QVector<BaseIndicatorPointer> indicators() const;
    BaseIndicatorPointer indicator(QModelIndex index) const;

    void addIndicators(QVector<BaseIndicator *> indicators, QVector<BaseIndicator *> &rejected);
    QModelIndex addIndicator(BaseIndicator *indicator);
    QModelIndex duplicateIndicator(BaseIndicator *indicator, bool uniqueName = false);
    QModelIndex duplicateIndicator(QModelIndex index, bool uniqueName = false);
    bool removeIndicator(const QModelIndex &index);

    void resetValues();
    void registerValues(const ABlock &block);
    void registerValues(const ItemMapper *mapper);

    QByteArray toXML() const;

    virtual Qt::DropActions supportedDropActions() const override;
    virtual Qt::DropActions supportedDragActions() const override;
    virtual QStringList mimeTypes() const override;
    virtual QMimeData *mimeData(const QModelIndexList &indexes) const override;
    virtual bool canDropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column,
                                 const QModelIndex &parent) const override;
    virtual bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column,
                              const QModelIndex &parent) override;

signals:

    /**
     * @brief Emitted when an item is removed.
     * @param item Removed item.
     * @details
     * The signal is only emitted when IndicatorModel::addItem is called.
     */
    void itemAdded(BaseIndicatorPointer item);

    /**
     * @brief Emitted when an item is about to be removed.
     * @param item Removed item.
     * @details
     * The signal is only emitted when IndicatorModel::removeItem is called.
     *
     */
    void itemAboutToBeRemoved(BaseIndicatorPointer item);

  private:
    QStringList mHeaders;
    QVector<BaseIndicator *> mItems;
};

#endif // INDICATORMODEL_HPP
