// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef LISTVIEW_HPP
#define LISTVIEW_HPP

#include <QKeyEvent>
#include <QListView>
#include <QMouseEvent>

/**
 * @brief The ListView class
 * @details
 * The class reimplements the mousePressEvent so a mouse press on an area that does not correspond to a valid index will
 * clear the selection (if any).
 */
class ListView : public QListView {
    Q_OBJECT
  public:
    ListView(QWidget *parent = nullptr) : QListView(parent) {}

    //virtual QSize sizeHint() const { return minimumSizeHint(); }

  protected:
    virtual void mousePressEvent(QMouseEvent *event) override {
        if (event->button() == Qt::LeftButton) {
            QModelIndex idx = indexAt(event->pos());
            if (!idx.isValid()) {
                if (selectionModel())
                    selectionModel()->clearSelection();
            }
        }
        QListView::mousePressEvent(event);
    }
};

#endif // LISTVIEW_HPP
