// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef MOVINGAVG_HPP
#define MOVINGAVG_HPP

#include <cmath>
#include <vector>

class MovingAverage {
  public:
    MovingAverage(std::size_t nSamples = 100);
    MovingAverage &push(double sample);
    void setTotalSamples(std::size_t n);
    double operator()() const;
    void reset();

  private:
    std::vector<double> mSamples;
    std::size_t mTotalSamples;
    std::size_t mNumSamples;
    double mTotal;
};

class ExponentialMovingAverage {
  public:
    ExponentialMovingAverage(double smoothing = 0.3);
    double operator()(double sample = NAN);
    void setSmoothing(double smoothing);
    void reset();

  private:
    double alpha;
    double last;
    long long int calls;
};

#endif // MOVINGAVG_HPP
