// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SCENARIO_HPP
#define SCENARIO_HPP

#include "globals.hpp"

#include "indicatormodel.hpp"
#include "itemmodel.hpp"

class Scenario : public QObject {
    Q_OBJECT

  public:
    explicit Scenario(QObject *parent = nullptr);
    Scenario(const QString &name, const QString &comments, const QVector<DataItem *> &items,
             const QVector<BaseIndicator *> &indicators, QObject *parent = nullptr);
    ~Scenario();

    static Scenario *copyFrom(Scenario *scenario, QObject *parent = nullptr);

    QString name() const;
    void setName(const QString &name);

    QString comments() const;
    void setComments(const QString &comments);

    int itemCount() const;
    int indicatorCount() const;

    QByteArray toXML() const;

    ItemModel *itemsModel() const;
    IndicatorModel *indicatorsModel() const;

    void resetValues();
    void registerValues(const ABlock &block);
    void registerValues(const ItemMapper *mapper);

  private:
    QString mName;
    QString mComments;
    ItemModel *mItemsModel;
    IndicatorModel *mIndicatorsModel;
};

#endif // SCENARIO_HPP
