// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SCENARIOEDITOR_HPP
#define SCENARIOEDITOR_HPP

#include "globals.hpp"

class LineEdit;
class Scenario;
class ScenariosModel;

class ScenarioEditor : public QWidget {
    Q_OBJECT
  public:
    explicit ScenarioEditor(QWidget *parent = nullptr);

    virtual ~ScenarioEditor();
    void translateUI();

    void setupScenarios(const QVector<Scenario *> scenarios, int *selectIndex = nullptr);
    int scenariosCount() const;
    QVector<Scenario *> scenarios() const;

    QPointer<Scenario> selectedScenario() const;
    int selectedScenarioIndex() const;

    int itemsCount(int scenarioIndex) const;

  signals:
    /**
     * @brief selectedScenarioChanged signal
     * @details
     * The signal is emitted whenever the selected scenario in the listview has changed. \a scenario
     * may be null if no scenario is selected in the view.
     */
    void selectedScenarioChanged(QPointer<Scenario> scenario);

  private slots:

    void addScenario();
    void duplicateScenario();
    void deleteScenario();
    void onEditingFinished();
    void onSelectionChanged(const QItemSelection &, const QItemSelection &deselected);

  private:
    void setupUI();
    QModelIndex selectedIndex() const;

  private:
    ScenariosModel *mScenarioModel;
    QAction *mActionAdd;
    QAction *mActionDuplicate;
    QAction *mActionDelete;
    QListView *mListView;
    LineEdit *mLdtName;
    QTextEdit *mTxeDescription;
};

#endif // SCENARIOEDITOR_HPP
