// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SCENARIOSEDITOR_HPP
#define SCENARIOSEDITOR_HPP

#include "globals.hpp"

class Scenario;
class ScenariosModel;
class ScenarioEditor;
class ItemsEditor;
class IndicatorsEditor;

/**
 * @brief The ScenariosEditor class
 * @details
 * The dialog allows to create removes and edits scenarios.
 *
 * Scenarios are can be given by ScenariosEditor::setupScenarios which copies the given vector of scenarios in the
 * internal model of the widget. After modifications, a copy of the scenarios can be retrieved using the method
 * ScenariosEditor::scenarios.
 */
class ScenariosEditor : public QDialog {

    Q_OBJECT
  public:
    explicit ScenariosEditor(QWidget *parent = nullptr);

    virtual ~ScenariosEditor();
    void translateUI();

    void setParameters(const QVector<AParam> &params);
    void setScenarios(const QVector<Scenario *> scenarios, int *selectIndex = nullptr);
    QVector<Scenario *> scenarios() const;
    QPointer<Scenario> selectedScenario() const;
    int selectedScenarioIndex() const;

    virtual QSize sizeHint() const override;

  protected:
    virtual void keyPressEvent(QKeyEvent *e) override;

  private slots:
    void onScenarioChanged(QPointer<Scenario> scenario);
    void sanityCheck();

  private:
    QSplitter *mSplitter;
    ScenarioEditor *mScenarioEditor;
    QTabWidget *mTabWidget;
    ItemsEditor *mItemsEditor;
    IndicatorsEditor *mIndicatorsEditor;
    QDialogButtonBox *mButtonBox;
};

#endif // SCENARIOSEDITOR_HPP
