// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SCENARIOSMODEL_HPP
#define SCENARIOSMODEL_HPP

#include "globals.hpp"

class Scenario;

typedef QPointer<Scenario> ScenarioPointer;

/**
 * @brief The ScenariosModel class
 * @details
 * A simple model to manage scenarios.
 *
 * The model does not own pointers to the internal scenarios. It is user responsibility to delete such pointers
 * when no more used.
 *
 */
class ScenariosModel : public QAbstractTableModel {
    Q_OBJECT
  public:
    ScenariosModel(QObject *parent = nullptr);
    virtual ~ScenariosModel();
    QVector<ScenarioPointer> scenarios() const;
    void setScenarios(const QVector<Scenario *> &scenarios);

    ScenarioPointer scenario(QModelIndex index) const;

    QModelIndex addScenario(Scenario *scenario);
    QModelIndex duplicateScenario(Scenario *scenario, bool uniqueName = false);
    QModelIndex duplicateScenario(QModelIndex index, bool uniqueName = false);
    bool removeScenario(int rowIndex);
    bool removeScenario(QModelIndex index);

    virtual Qt::ItemFlags flags(const QModelIndex &index) const override;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    void translateHeaders();

    virtual Qt::DropActions supportedDropActions() const override;
    virtual Qt::DropActions supportedDragActions() const override;
    virtual QStringList mimeTypes() const override;
    virtual QMimeData *mimeData(const QModelIndexList &indexes) const override;
    virtual bool canDropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column,
                                 const QModelIndex &parent) const override;
    virtual bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column,
                              const QModelIndex &parent) override;

signals:
    void scenarioAboutToBeDeleted(QPointer<Scenario> scenario);

  private:
    QStringList mHeaders;
    QVector<Scenario *> mScenarios;
};

#endif // SCENARIOSMODEL_HPP
