// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SETTINGSDIALOG_HPP
#define SETTINGSDIALOG_HPP

#include "globals.hpp"
class BasePage;

class PageModel : public QAbstractListModel {
  public:
    PageModel(QObject *parent = nullptr);
    ~PageModel() override;

    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    void setPages(const QVector<BasePage *> &pages);
    BasePage *page(int index);
    BasePage *page(QModelIndex index);

    const QVector<BasePage *> &pages() const;

  private:
    QVector<BasePage *> mPages;
    QIcon mEmptyIcon;
};

class SettingsDialog : public QDialog {
    Q_OBJECT
  public:
    explicit SettingsDialog(QWidget *parent);

    void showPage(int pageId);

    void translateUI();
    virtual QSize sizeHint() const final; // starts up as small as possible

  public slots:
    virtual void done(int) final;

  signals:
    void settingsChanged();

  protected:
    virtual void changeEvent(QEvent *event) override;

  private slots:
    void manageButtonBox(QAbstractButton *btn);
    void currentChanged(const QModelIndex &current);

  private:
    void loadSettings();
    void saveSettings();

    void setupUI();

    bool mHasUISetting;
    QLabel *mCategoryLabel;
    QLabel *mHeaderLabel;
    PageModel mModel;

    QStackedLayout *mStackedLayout;
    QVector<BasePage *> mPages;
    QSet<BasePage *> mVisitedPages;
    QListView *mPageList;
    QTextBrowser *mInfoWidget;
    QDialogButtonBox *mButtonBox;
};

#endif // SETTINGSDIALOG_HPP
