// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef STYLEPAGE_HPP
#define STYLEPAGE_HPP

#include "globals.hpp"
#include "basepage.hpp"

class ColorChooserLabel;

class MapUIWidget : public QWidget {

    Q_OBJECT
  public:
    explicit MapUIWidget(QWidget *parent = nullptr);
    void loadSettings();
    void saveSettings();
    void translateUI();

  private:
    QCheckBox *mChbMapGrid;
    QCheckBox *mChbHoriCB;
    QCheckBox *mChbHideMV;

    QGroupBox *mGbxMissingValue;

    QLabel *mLblMVColor;
    ColorChooserLabel *mLblMVColorChooser;
    QLabel *mLblMVStyle;
    QComboBox *mCbxMVStyle;
    QDoubleSpinBox *mSpbMVSize;

    QGroupBox *mGbxCurrentFlight;
    QLabel *mLblCFStyle;
    QComboBox *mCbxCFStyle;
    QDoubleSpinBox *mSpbCFSize;

    QGroupBox *mGbxPreviousFlights;
    QLabel *mLblPFStyle;
    QComboBox *mCbxPFStyle;
    QDoubleSpinBox *mSpbPFSize;

  private:
    void buildUI();
};

class GraphUIWidget : public QWidget {
    Q_OBJECT
  public:
    explicit GraphUIWidget(QWidget *parent = nullptr);
    void loadSettings();
    void saveSettings();
    void translateUI();
  private slots:
    void updatePlotWidgets(int id, bool checked);

  private:
    QButtonGroup *mButtonGroup;
    QLabel *mLblPlotType;
    QRadioButton *mBtnPLine;
    QRadioButton *mBtnPScatter;
    QLabel *mLblPLineSize;
    QLabel *mLblPSymbol;
    QDoubleSpinBox *mSpbPLineSize;
    QComboBox *mCbxPSymbol;
    QDoubleSpinBox *mSpbPSymbolSize;
    int mLastLMS;
    int mLastSMS;

  private:
    void buildUI();
};

class StylePage : public BasePage {
    Q_OBJECT
  public:
    explicit StylePage(QWidget *parent = nullptr);
    virtual void loadSettings() final;
    virtual void saveSettings() final;
    virtual void translateUI() final;
    virtual void saveUI() override;
    virtual void loadUI() override;
  private:
    void buildUI();

    QTabWidget *mTabWidget;

    GraphUIWidget *mPlotWidget;
    MapUIWidget *mMapWidget;
};

#endif // STYLEPAGE_HPP
