// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#include "warningspage.hpp"

/// @brief Constructor
WarningsPage::WarningsPage(QWidget *parent) : BasePage(parent) {
    mPageTitle = tr("Warnings");
    mPageIcon = getIcon("warnings");
    buildUI();
    translateUI();
}

/// @brief Initialize widgets of the General group box.
void WarningsPage::buildUI() {
    mChbArcGISSignin = new QCheckBox(this);
    mChbConfChange = new QCheckBox(this);
    mChbBadFitConf = new QCheckBox(this);
    mChbEmptyScn = new QCheckBox(this);
    mChbMapLoadFail = new QCheckBox(this);
    mChbReplaySwitch = new QCheckBox(this);
    mChbNoSD = new QCheckBox(this);

    QVBoxLayout *layout = new QVBoxLayout(this);
    layout->addWidget(mChbArcGISSignin);
    layout->addWidget(mChbConfChange);
    layout->addWidget(mChbBadFitConf);
    layout->addWidget(mChbEmptyScn);
    layout->addWidget(mChbMapLoadFail);
    layout->addWidget(mChbReplaySwitch);
    layout->addWidget(mChbNoSD);
    layout->addStretch(0);
}

/// @brief Load general settings from configuration file.
void WarningsPage::loadSettings() {
    QSettings settings;
    settings.beginGroup("warnings");
    mChbArcGISSignin->setChecked(settings.value("no_arcgis_signin", true).toBool());
    mChbConfChange->setChecked(settings.value("conf_change", true).toBool());
    mChbBadFitConf->setChecked(settings.value("bad_fit_cfg", true).toBool());
    mChbEmptyScn->setChecked(settings.value("empty_scn", true).toBool());
    mChbMapLoadFail->setChecked(settings.value("map_load_fail", true).toBool());
    mChbReplaySwitch->setChecked(settings.value("replay_switch", true).toBool());
    mChbNoSD->setChecked(settings.value("no_sd_detected", true).toBool());
    settings.endGroup();
}

/// @brief Save general settings in configuration file.
void WarningsPage::saveSettings() {
    QSettings settings;
    settings.beginGroup("warnings");
    settings.setValue("no_arcgis_signin", mChbArcGISSignin->isChecked());
    settings.setValue("conf_change", mChbConfChange->isChecked());
    settings.setValue("bad_fit_cfg", mChbBadFitConf->isChecked());
    settings.setValue("empty_scn", mChbEmptyScn->isChecked());
    settings.setValue("map_load_fail", mChbMapLoadFail->isChecked());
    settings.setValue("replay_switch", mChbReplaySwitch->isChecked());
    settings.setValue("no_sd_detected", mChbNoSD->isChecked());
    settings.endGroup();
}

/// @brief Translates widgets of the general groupbox.
void WarningsPage::translateUI() {
    mPageTitle = tr("Warnings");
    mChbArcGISSignin->setText(tr("Prompt user for ArGIS sign-in on startup."));
    mChbConfChange->setText(tr("Warn user on configuration changes."));
    mChbBadFitConf->setText(tr("Warn user about invalid fit configuration."));
    mChbEmptyScn->setText(tr("Warn user about empty scenarios on edition finished."));
    mChbMapLoadFail->setText(tr("Warn user on basemap loading failures."));
    mChbReplaySwitch->setText(tr("Warn user before switching to replay mode."));
    mChbNoSD->setText(tr("Warn user when no SD card has been detected in the sensor."));

    setHelper(mChbArcGISSignin, HelpFlag::WARN_ARCGIS_SIGNIN);
    setHelper(mChbConfChange, HelpFlag::WARN_CONF);
    setHelper(mChbEmptyScn, HelpFlag::WARN_EMPTY_SCN);
    setHelper(mChbMapLoadFail, HelpFlag::WARN_LMFAIL);
    setHelper(mChbReplaySwitch, HelpFlag::WARN_REPLAY);
    setHelper(mChbNoSD, HelpFlag::WARN_NOSD);
}
