// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef TIMESTAMPWIDGET_HPP
#define TIMESTAMPWIDGET_HPP

#include "globals.hpp"

class ItemMapper;
class ColorIndicator;

/**
 * @brief The TimestampWidget class
 * @details
 * The widget displays the following timestamps:
 *
 * - On board, GPS time of Adafruit module.
 * - On board, system time.
 * - Monitoring PC system time.
 *
 * Values are updated from the associated item mapper by calling TimestampWidget::updateFromMapper.
 */
class TimestampWidget : public QWidget {
    Q_OBJECT
  public:
    explicit TimestampWidget(QWidget *parent = nullptr);
    void setMapper(ItemMapper *mapper);
    void setWarmupTime(int milliseconds);
    void reset();
    void translateUI();

    virtual QSize sizeHint() const override;

  public slots:
    void updateFromMapper();

  signals:

  private:
    QFormLayout *mLayout;
    // timestamp labels
    QLabel *mLblHSysDescr;
    QLabel *mLblHGpsDescr;
    QLabel *mLblHAcqDescr;
    QLabel *mLblHSysValue;
    QLabel *mLblHGpsValue;
    QLabel *mLblHAcqValue;

    QColor mHSysColor;

    // elapsed time & laser warmup
    QLabel *mLblAETDescr;
    QLabel *mLblAETValue;
    QLabel *mLblWarmupDescr;
    ColorIndicator *mWarmupIndicator;
    int mWarmupTime;
    QPointer<ItemMapper> mMapper;
    void buildTimeStampLabels();
    void buildAdditionalLabels();
    void updateHSysColor();
};

#endif // TIMESTAMPWIDGET_HPP
