// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

/**
 * @file scilineedit.hpp
 * @brief Science double editor class declarations file.
 * @author J. Burgalat
 * @date 2021
 */
#ifndef VALUELINEEDIT_H
#define VALUELINEEDIT_H

#include <QLineEdit>

#include <QString>
#include <QVariant>

class DoubleValidator;

/**
 * @brief The ScienceEdit class
 * @details
 * QLineEdit subclass oriented for floating point value display that account for optional
 * optional range constraint.
 *
 * The class, derived from QLineEdit, allows to set a range of validity (similar to QDoubleSpinBox).
 * Although the range is set using double values, those are internally converted in the nearest"
 * string representation.
 *
 * Range check (if any) are performed on the string value of the QLineEdit, not on derived double
 *value.
 *
 */
class ValueLineEdit : public QLineEdit {
    Q_OBJECT

  public:
    ValueLineEdit(bool allowEmpty, bool allowNaN, QWidget *parent = nullptr);
    virtual ~ValueLineEdit() {}
    void setAnimation(bool enabled);

  protected:
    virtual void focusOutEvent(QFocusEvent *event) override;
    virtual void focusInEvent(QFocusEvent *event) override;
    virtual void mousePressEvent(QMouseEvent *event) override;

  private:
    DoubleValidator *mValidator;
    QString mLast;
    static const QString mNaN;
    bool mNoAnim;
    bool mJustFocused;
    bool mAllowEmpty;
    bool mAllowNaN;
};

#endif // VALUELINEEDIT_H
