// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef MAINWINDOW_HPP
#define MAINWINDOW_HPP

#include "csvfilewriter.hpp"
#include "datasetinfowidget.hpp"
#include "globals.hpp"
#include "logwidget.h"
#include "prmchoicewidget.hpp"
#include "progressbar.hpp"

class MainWindow : public QMainWindow {
    Q_OBJECT

  public:
    MainWindow(QWidget *parent = nullptr);
    virtual ~MainWindow() override;
    virtual QSize sizeHint() const override;
    void translateUI();

  protected:
    virtual void changeEvent(QEvent *event) override;
    virtual void closeEvent(QCloseEvent *event) override;

    virtual void dropEvent(QDropEvent *event) override;
    virtual void dragMoveEvent(QDragMoveEvent *event) override;
    virtual void dragEnterEvent(QDragEnterEvent *event) override;

  private slots:
    void convertParameters();
    void onItemMoved();
    void cancelOperations();
    void onProgressChanged(int value);
    void onProgressRangeChanged(int minValue, int maxValue);
    void onLoadingFinished(bool aborted);
    void onDatasetLoaded(AMDataset *dataset);
    void onDataConverted(bool aborted);
    void openDirectory();
    void openFiles();
    void aboutApp();
    void openHelp();
    void enableDebugLog(bool enable);
    void logMessage(const QString &message);

  private:
    QMap<QString, QVector<double>> mData;

    LogWidget *mLogWidget;
    DatasetInfoWidget *mDatasetInfoWidget;
    PrmChoiceWidget *mPrmChoiceWidget;
    QCheckBox *mChbSaveChanges;
    QLabel *mLblPrecision;
    QSpinBox *mSpbPrecision;
    QPushButton *mBtnConvert;

    StatusProgressBar *mStatusPBar;
    ProgressBar *mProgressBar;

    XMLDContainer mXMLDContainer;
    XMLDescriptor *mXMLDCurrent;
    int mInstrument;
    int mVersion;
    bool mFirst;

    QMenu *mMenuFiles;
    QMenu *mMenuSubFiles;
    QMenu *mMenuView;
    QMenu *mMenuHelp;
    QAction *mActionOpen;
    QAction *mActionOpenDirectory;
    QAction *mActionAbout;
    QAction *mActionDebug;
    QAction *mActionQuit;
#ifndef NO_HELP
    QAction *mActionHelp;
#endif

    bool mDebug;

    QThreadPool *mThreadPool;

  private:
    void loadUI();
    void saveUI();
    void createActions();
    void checkAndLoad(const QStringList &files);
    void loadFiles(const QVector<QStringList> &paths);
    void filterListOfPaths(const QStringList &inputs);
    void saveDescriptor();
    PairOfData dataSelection(QVector<PairOfStrings> pairs);

  signals:
    void abortTasks();
};

#endif // MAINWINDOW_HPP
