// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef XMLDESCRIPTORS_HPP
#define XMLDESCRIPTORS_HPP

#include "globals.hpp"
#include <QtCore>
#include <QDomDocument>
#include "isp/aparam.hpp"

using namespace ISP;

class XMLDescriptor {
  public:
    int instrument;
    int version;
    QVector<PairOfStrings> parameters;
    QStringList selection;
    QString inputPath;

    XMLDescriptor();

    XMLDescriptor(const XMLDescriptor &other);
    XMLDescriptor &operator=(const XMLDescriptor &other);
    QByteArray toXML(bool pretty = false) const;
    QString fileName() const;
    bool hasParameter(const QString &name);
    PairOfStrings pop(const QString &name);
    void setAlias(const QString &name, const QString &alias);
};

class XMLDContainer {

  public:
    XMLDContainer();
    virtual ~XMLDContainer();
    int setupFromDirectory(const QString &path);
    XMLDescriptor *get(int instrument, int version, const QVector<AParam> &params = QVector<AParam>());
    QByteArray xsdSchema() const;
    void clear();

  private:
    QVector<XMLDescriptor *> mDescriptors;
    QByteArray mXsdSchema;

  private:
    bool parse(const QString &xmlPath, QStringList *errors = nullptr);
    bool parseContent(const QString &content, const QString *path = nullptr, QStringList *errors = nullptr);
    bool validate(const QString &xmlPath, QStringList *errors = nullptr);
    bool validateContent(const QString &xmlContent, QStringList *errors = nullptr);
    bool readDescriptor(QDomElement parent, QString *path = nullptr);
    PairOfStrings readParameter(QDomElement parent);
};

#endif // XMLDESCRIPTORS_HPP
