// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024

#ifndef SIMPLEFILELOGGER_HPP
#define SIMPLEFILELOGGER_HPP

#include <QtCore>

class SimpleFileLogger {
  public:
    /**
     * @brief Constructor.
     * @param path Path of the log file.
     * @param prependDate True to prepend datetime to the message, false otherwise. Default to false.
     */
    SimpleFileLogger(const QString &path, bool prependDate = false) : mFile(nullptr), mStream(nullptr), mWithDate(prependDate) {
        if (path.compare("__stdout__") == 0) {
            mStream = new QTextStream(stdout);
        } else if (path.compare("__stderr__") == 0) {
            mStream = new QTextStream(stderr);
        } else if (path.size()) {
            QFileInfo finfo(path);
            QDir dinfo(finfo.absolutePath());
            if (!dinfo.exists())
                dinfo.mkpath(".");
            mFile = new QFile(path);
            if (!mFile->open(QIODevice::Text | QIODevice::WriteOnly)) {
                delete mFile;
                mFile = nullptr;
            } else {
                mStream = new QTextStream(mFile);
            }
        }
    }

    /// @brief Destructor.
    ~SimpleFileLogger() { close(); }

    /// @brief Write message to file.
    void write(const QString &message) {
        if (!mStream)
            return;
        if (mWithDate)
            (*mStream) << QDateTime::currentDateTime().toString("dd/MM/yyyy hh:mm:ss.zzz : ");
        (*mStream) << message;
    }

    ///  @brief Close the internal file.
    void close() {
        if (mStream) {
            delete mStream;
            mStream = nullptr;
        }
        // we have a file.
        if (mFile) {
            mFile->close();
            delete mFile;
            mFile = nullptr;
        }
    }

  private:
    QFile *mFile;
    QTextStream *mStream;
    bool mWithDate;
};

#endif // SIMPLEFILELOGGER_HPP
