// Copyright (c) URCA-CNRS-TOTALENERGIES - All Rights Reserved
// Authors: J. Burgalat
// 2016-2024
#ifndef FILELOADER_HPP
#define FILELOADER_HPP

#include <QtCore>

class AMDataset;


/// @brief Load files into dataset.
class FileLoader : public QObject {
    Q_OBJECT
  public:
    FileLoader(const QStringList &files, QObject *parent = nullptr);
    virtual ~FileLoader();
    void setFilesToLoad(const QStringList &files);
    bool canceled() const;
    bool isRunning() const;

signals:
    /**
     * @brief Emitted when the dataset is available.
     * @param dataset Pointer to the dataset.
     * @details
     * The pointer is not owned by this loader. It user responsibility to delete it if needed.
     * @note
     * The signal is emited only if the job has not been canceled and at least one of files have been successfully processed.
     */
    void datasetLoaded(AMDataset *dataset);
    /**
     * @brief Emitted when job is finished whatever the status is.
     * @param aborted True if job has been aborted, false otherwise.
     */
    void finished(bool aborted = false);
    /// @brief Emitted whenever the total progress range changes.
    void progressRangeChanged(int min, int max);
    /// @brief Emitted whenever progress changed.
    void progressChanged(int value);
    /// @brief Emitted whenever a message is sent by the loader.
    void message(const QString msg);
    /// @brief Emitted whenever a debug message is sent by the loader.
    void debugMessage(const QString msg);
  public slots:
    void process();
    void cancel();

  private:
    mutable QMutex mDataMutex;
    bool mIsRunning;
    bool mCancel;
    QStringList mFilesToLoad;
};

#endif // FILELOADER_HPP
